﻿using System;
using System.IO;
using System.Reflection;
using UnityEngine;

public class UnityExplorer : Mod
{
    public void Start()
    {
        string appDataDirectory = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "UnityExplorerRaft");
        if (!Directory.Exists(appDataDirectory)) Directory.CreateDirectory(appDataDirectory);

        string[] files = new string[]
        {
            "UnityExplorer.STANDALONE.Mono.dll",
            "UniverseLib.Mono.dll",
            "UniverseLib.Mono.xml"
        };

        foreach(string file in files)
        {
            string from = file + ".copyonly";
            string to = Path.Combine(appDataDirectory, file);

            if (!CopyDLL(from, to))
            {
                Debug.LogError("An error occured while copying UnityExplorer libraries.");
                return;
            }
        }

        Assembly assembly = Assembly.LoadFrom(Path.Combine(appDataDirectory, files[0]));

        Type type = assembly.GetType("UnityExplorer.ExplorerStandalone");

        type.GetMethod("CreateInstance", new Type[] { }).Invoke(null, null);

        Debug.Log("Mod UnityExplorer has been loaded!");
    }

    private bool CopyDLL(string from, string to)
    {
        if (File.Exists(to)) return true;
        try
        {
            byte[] assemblyBytes = GetEmbeddedFileBytes(from);
            if (assemblyBytes != null && assemblyBytes.Length > 0)
            {
                Debug.Log("Copying " + from + "...");
                File.WriteAllBytes(to, assemblyBytes);
                return true;
            }
            else
            {
                return false;
            }
        }
        catch (Exception)
        {
            return false;
        }
    }

    public void OnModUnload()
    {
        Debug.Log("Mod UnityExplorer has been unloaded! However, the UnityExplorer UI won't disappear until you close it with F7 or close the game.");
    }
}